function RandDrift(cyclespersecond,amplitude,onTime, offTime, frequencies,angles,iti)
% function RandDrift([cyclespersecond=2][, amplitude=0.5]
%[, onTime=1.5][, offTime=0.200]
%[, frequencies=[0 0.155 0.309 0.618 1.237 2.437 4.947]]
%[, angles=[0 30 60 90 120 150 180 210 240 270 300 330]])
%[, iti=2.0]
% ___________________________________________________________________
% RandDrift aka SineGratings Presents a series of sinusoidal stimuli at various angles and
% spacial frequencies in random order.
%
%   This function takes every combination of frequencies and angles from
%   the frequencies and angles arrays and presents them in random order
%   for a stimulus interval (1.5s default) with an interstimulus interval
%   of solid gray (200ms default).  Marker codes are sent to a Spike2
%   system using Nidaq PCI-6503 hardware just after the end of the first frame of a
%   stimulus presentation and again just after the last frame of that presentation.  Markers sent
%   to spike2 are indeces representing the selected sinusoidal stimulus.
%   The first 0 through numberOfAngles-1 indeces represent frequency 0,
%   angles 0 through numberOfAngles-1, the second numberOfAngles indeces represent
%   frequency 1 angles 0 through numberOfAngles-1 and so on.
%   The presentation begins with a gray screen.  This gray screen will
%   persist until any key on the keyboard is pressed, then released.  At
%   that point stimulus presentation will begin.  Stimulus presentation
%   continues until the user presses a key on the keyboard.  Once all 
%   stimuli have been presented, a short intertrial interval gray screen
%   will be presented, the frequency/angle combination order will be
%   re-randomized and stimuli presentation will begin again.
%   
%Parameters:
%   cyclespersecond         -   Cycles of sinusoid that should pass a
%                               pixel per second
%   amplitude               -   Amplitude of sinusoid intensity (.5
%                               indicates full display intensity range).
%   onTime                  -   Time to display sinusoidal stimulus
%                               (seconds)
%   offTime                 -   Time to display solid gray screen (seconds)
%
%   frequencies             -   Array of sinusoid frequencies.  Units are
%                               in degrees per pixel
%   angles                  -   Angles of sinusoidal stimulus display in
%                               degrees
%   iti                     -   Intertrial interval.  Time to display solid
%                               gray screen between full frequency/stimuli
%                               sets. (seconds)
% History:
% 11/2/2011  Joey Schnurr   Written for Ethan Mohns Crair Lab at Yale 
% 11/18/2011  Joey Schnurr  Corrected bug that prevented end stimulus 
%                           marker presentation.
% DriftDemo4_1.m

%Setup default values------------------------------------------------------
if nargin < 7 || isempty(iti)
    % Define array of grating angles (units are degrees)
    iti = 2.0;
end
if nargin < 6 || isempty(angles)
    % Define array of grating angles (units are degrees)
    angles = [0 30 60 90 120 150 180 210 240 270 300 330];
end
if nargin < 5 || isempty(frequencies)
    % Define array of grating frequencies (units are degrees per pixel)
    frequencies = [0 0.155 0.309 0.618 1.237 2.437 4.947];
end
if nargin < 4 || isempty(offTime)
    % Define inter stimulus interval time
    offTime = 0.200;
end
if nargin < 3 || isempty(onTime)
    % Define time for stimulus presentation
    onTime = 1.5;
end
if nargin < 2 || isempty(amplitude)
    % Define amplitude of the grating in units of absolute display intensity range: A
    % setting of 0.5 means that the grating will extend over a range from -0.5
    % up to 0.5, i.e., it will cover a total range of 1.0 == 100% of the total
    % displayable range. As we select a background color and offset for the
    % grating of 0.5 (== 50% nominal intensity == a nice neutral gray), this
    % will extend the sinewaves values from 0 = total black in the minima of
    % the sine wave up to 1 = maximum white in the maxima. Amplitudes of more
    % than 0.5 don't make sense, as parts of the grating would lie outside the
    % displayable range for your computers displays:
    amplitude = 0.5;
end

if nargin < 2 || isempty(cyclespersecond)
  % Define number of cycles that should move past a pixel per second
    cyclespersecond = 2;  
end








%Initialize local variables and devices------------------------------------

% Choose screen with max id - the secondary display on a dual-display
% setup for display:
screenid = max(Screen('Screens'));

% Open a fullscreen onscreen window on that display, choose a background
% color of 128 = gray, i.e. 50% max intensity:
win = Screen('OpenWindow', screenid, 128);

% Calculate largest line that can be drawn on the screen
[width, height]=Screen('WindowSize', win);
maxLine = round((width^2+height^2)^.5)+1;

% res is the total size of the patch in x- and y- direction, i.e., the
% width and height of the mathematical support:
res = [maxLine maxLine];

% Setup Digital I/O Object
hasNidaq = true;
dio = digitalio('nidaq','Dev1');
hwlines = addline(dio,0:15,'out');

% Make sure this is running on OpenGL Psychtoolbox:
AssertOpenGL;

% Make sure the GLSL shading language is supported:
AssertGLSL;

% Retrieve video redraw interval for later control of our animation timing:
ifi = Screen('GetFlipInterval', win);

% Compute increment of phase shift per redraw:
phaseincrement = (cyclespersecond * 360) * ifi;

% Build a procedural sine grating texture for a grating with a support of
% res(1) x res(2) pixels and a RGB color offset of 0.5 -- a 50% gray.
gratingtex = CreateProceduralSineGrating(win, res(1), res(2), [0.5 0.5 0.5 0.0]);

% Wait for start of experiment---------------------------------------------

% Initial gray Screen.  Repeats until keypress...
vbl = Screen('Flip', win);
while ~KbCheck 
    % Show it at next retrace:
    vbl = Screen('Flip', win, vbl + 0.5 * ifi);     
end
while KbCheck 
    % Show it at next retrace:
    vbl = Screen('Flip', win, vbl + 0.5 * ifi);
end
vbl = Screen('Flip', win);

% Loop until done (done will be set by the detection of a keypress) -------
done = false;
while(~done)
    % Create an array of sample indeces
    freqLength = length(frequencies);
    angLength = length(angles);
    indeces = 0:(freqLength*angLength)-1;
    totInd = length(indeces);
    % Randomly shuffle it to get our presentation order
    indeces = Shuffle(indeces);

    % Main Loop----------------------------------------------------------------

    for i = 0:totInd-1
        index = indeces(i+1);
        angInd = 1+mod(index,angLength); % index of 0-angLength-1 = freq:0, angle: index, angLength-2*angLength-1 = freq:1, angle: index-angLength
        freqInd = 1+(index-angInd+1)/angLength;
        freq = frequencies(freqInd)/360;
        angle = angles(angInd);

        % Phase is the phase shift in degrees (0-360 etc.)applied to the sine grating:
        phase = 0;

        triggeredoutput = bitand(index,32767); % =54 & 0x7FFF
        untriggeredoutput = bitor(index,32768); % =54 & 0x8000

        numOnFrames = onTime/ifi;
        % Animation loop: Repeats for OnTime...
        for k = 1:numOnFrames
            % On keypress, End
            if(KbCheck)
                done = true;
                break;
            end
            % Update some grating animation parameters:

            % Increment phase by 1 degree:
            phase = phase + phaseincrement;

            % Draw the grating, centered on the screen, with given rotation 'angle',
            % sine grating 'phase' shift and amplitude.
            % Note that we pad the last argument with a 4th
            % component, which is 0. This is required, as this argument must be a
            % vector with a number of components that is an integral multiple of 4,
            % i.e. in our case it must have 4 components:
            Screen('DrawTexture', win, gratingtex, [], [], angle, [], [], [], [], [], [phase, freq, amplitude, 0]);

            % Show it at next retrace:
            vbl = Screen('Flip', win, vbl + 0.5 * ifi);
            if(hasNidaq)
                if(k==1)
                    %   Send marker just after first frame
                    try
                        putvalue(dio,triggeredoutput);
                    catch err
                        hasNidaq = false;
                        msg = sprintf('%s', ...
                       'Could not send stimulus markers with National Instruments hardware.  Device is not available.');
                        warning('MATLAB:RandDrift:putvalue', msg);
                    end
                    if(hasNidaq)
                        % Spike2 expects 1 us, so hopefully this isn't under a us
                        putvalue(dio,untriggeredoutput);
                    end
                end
                if(k+1>numOnFrames)
                    %   Send marker just after last frame
                    putvalue(dio,triggeredoutput);
                    % Spike2 expects 1 us, so hopefully this isn't under a us
                    putvalue(dio,untriggeredoutput);
                end
            end
        end

        numOffFrames = offTime/ifi; 
        % InterTrialInverval loop.  Repeats for offTime.
        for k = 1:numOffFrames
            % On keypress, End
            if(KbCheck)
                done = true;
                break;
            end
            % Show it at next retrace:
            vbl = Screen('Flip', win, vbl + 0.5 * ifi);
        end

    end
    
    % Show gray in between each main loop
    numOffFrames = iti/ifi; 
    % InterTrialInverval loop.  Repeats for offTime.
    for k = 1:numOffFrames
        % On keypress, End
        if(KbCheck)
            done = true;
            break;
        end
        % Show it at next retrace:
        vbl = Screen('Flip', win, vbl + 0.5 * ifi);
    end
end
% All done.  Delete the digital I/O object.
delete (dio);
clear dio;

% We're done. Close the window. This will also release all other ressources:
Screen('CloseAll');

% Bye bye!
return;

